# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

class SystematicUnits(object):
    syst_unspecified = 0
    syst_confidence = 1
    syst_sigmas = 2

