# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

class ObjectData(object):
    NONE = 0
    Histogram = 1
    Ntuple = 2
    FunctionObject = 3
    Collection = 4

