# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

# ///////////////////////////////////////////////// metadata
class MetadataLanguage(object):
    meta_unspecified = 0
    meta_json = 1

