# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

class Binning(object):
    NONE = 0
    IntegerBinning = 1
    RegularBinning = 2
    HexagonalBinning = 3
    EdgesBinning = 4
    IrregularBinning = 5
    CategoryBinning = 6
    SparseRegularBinning = 7
    FractionBinning = 8
    PredicateBinning = 9
    VariationBinning = 10

