# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

# ///////////////////////////////////////////////// binning
class BinLocation(object):
    loc_below3 = -3
    loc_below2 = -2
    loc_below1 = -1
    loc_nonexistent = 0
    loc_above1 = 1
    loc_above2 = 2
    loc_above3 = 3

