# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

class NonRealMapping(object):
    missing = 0
    in_underflow = 1
    in_overflow = 2
    in_nanflow = 3

