# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

# ///////////////////////////////////////////////// decoration
class DecorationLanguage(object):
    deco_unspecified = 0
    deco_css = 1
    deco_vega = 2
    deco_json = 3

