# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

class FractionErrorMethod(object):
    frac_unspecified = 0
    frac_normal = 1
    frac_clopper_pearson = 2
    frac_wilson = 3
    frac_agresti_coull = 4
    frac_feldman_cousins = 5
    frac_jeffrey = 6
    frac_bayesian_uniform = 7

