# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

class OverlappingFillStrategy(object):
    overfill_unspecified = 0
    overfill_all = 1
    overfill_first = 2
    overfill_last = 3

