# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

class FunctionObjectData(object):
    NONE = 0
    ParameterizedFunction = 1
    BinnedEvaluatedFunction = 2

