# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

# ///////////////////////////////////////////////// buffers
class DType(object):
    dtype_none = 0
    dtype_bool = 1
    dtype_int8 = 2
    dtype_uint8 = 3
    dtype_int16 = 4
    dtype_uint16 = 5
    dtype_int32 = 6
    dtype_uint32 = 7
    dtype_int64 = 8
    dtype_uint64 = 9
    dtype_float32 = 10
    dtype_float64 = 11

